@@ -1,11 +1,10 @@
 # -*- coding: utf-8 -*-
 # Description: redis netdata python.d module
 # Author: Pawel Krupa (paulfantom)
-# Author: Ilya Mashchenko (l2isbad)
+# Author: Ilya Mashchenko (ilyam8)
 # SPDX-License-Identifier: GPL-3.0-or-later
 
 import re
-
 from copy import deepcopy
 
 from bases.FrameworkServices.SocketService import SocketService
@@ -37,7 +36,6 @@ PIKA_ORDER = [
     'uptime',
 ]
 
-
 CHARTS = {
     'operations': {
         'options': [None, 'Operations', 'operations/s', 'operations', 'redis.operations', 'line'],
@@ -47,13 +45,13 @@ CHARTS = {
         ]
     },
     'hit_rate': {
-        'options': [None, 'Hit rate', 'percent', 'hits', 'redis.hit_rate', 'line'],
+        'options': [None, 'Hit rate', 'percentage', 'hits', 'redis.hit_rate', 'line'],
         'lines': [
             ['hit_rate', 'rate', 'absolute']
         ]
     },
     'memory': {
-        'options': [None, 'Memory utilization', 'kilobytes', 'memory', 'redis.memory', 'line'],
+        'options': [None, 'Memory utilization', 'KiB', 'memory', 'redis.memory', 'line'],
         'lines': [
             ['used_memory', 'total', 'absolute', 1, 1024],
             ['used_memory_lua', 'lua', 'absolute', 1, 1024]
@@ -62,8 +60,8 @@ CHARTS = {
     'net': {
         'options': [None, 'Bandwidth', 'kilobits/s', 'network', 'redis.net', 'area'],
         'lines': [
-            ['total_net_input_bytes', 'in', 'incremental', 8, 1024],
-            ['total_net_output_bytes', 'out', 'incremental', -8, 1024]
+            ['total_net_input_bytes', 'in', 'incremental', 8, 1000],
+            ['total_net_output_bytes', 'out', 'incremental', -8, 1000]
         ]
     },
     'keys_redis': {
@@ -146,19 +144,17 @@ RE = re.compile(r'\n([a-z_0-9 ]+):(?:keys=)?([^,\r]+)')
 class Service(SocketService):
     def __init__(self, configuration=None, name=None):
         SocketService.__init__(self, configuration=configuration, name=name)
-        self._keep_alive = True
-
         self.order = list()
         self.definitions = dict()
-
+        self._keep_alive = True
         self.host = self.configuration.get('host', 'localhost')
         self.port = self.configuration.get('port', 6379)
         self.unix_socket = self.configuration.get('socket')
         p = self.configuration.get('pass')
-
         self.auth_request = 'AUTH {0} \r\n'.format(p).encode() if p else None
         self.request = 'INFO\r\n'.encode()
         self.bgsave_time = 0
+        self.keyspace_dbs = set()
 
     def do_auth(self):
         resp = self._get_raw_data(request=self.auth_request)
@@ -192,23 +188,38 @@ class Service(SocketService):
         :return: dict
         """
         data = self.get_raw_and_parse()
-
         if not data:
             return None
 
+        self.calc_hit_rate(data)
+        self.calc_redis_keys(data)
+        self.calc_redis_rdb_save_operations(data)
+        return data
+
+    @staticmethod
+    def calc_hit_rate(data):
         try:
-            data['hit_rate'] = (
-                    (int(data['keyspace_hits']) * 100) / (int(data['keyspace_hits']) + int(data['keyspace_misses']))
-            )
+            hits = int(data['keyspace_hits'])
+            misses = int(data['keyspace_misses'])
+            data['hit_rate'] = hits * 100 / (hits + misses)
         except (KeyError, ZeroDivisionError):
             data['hit_rate'] = 0
 
-        if data.get('redis_version') and data.get('rdb_bgsave_in_progress'):
-            self.get_data_redis_specific(data)
+    def calc_redis_keys(self, data):
+        if not data.get('redis_version'):
+            return
+        # db0:keys=2,expires=0,avg_ttl=0
+        new_keyspace_dbs = [k for k in data if k.startswith('db') and k not in self.keyspace_dbs]
+        for db in new_keyspace_dbs:
+            self.keyspace_dbs.add(db)
+            self.charts['keys_redis'].add_dimension([db, None, 'absolute'])
+        for db in self.keyspace_dbs:
+            if db not in data:
+                data[db] = 0
 
-        return data
-
-    def get_data_redis_specific(self, data):
+    def calc_redis_rdb_save_operations(self, data):
+        if not (data.get('redis_version') and data.get('rdb_bgsave_in_progress')):
+            return
         if data['rdb_bgsave_in_progress'] != '0':
             self.bgsave_time += self.update_every
         else:
@@ -232,11 +243,6 @@ class Service(SocketService):
         for n in self.order:
             self.definitions.update(copy_chart(n))
 
-        if data.get('redis_version'):
-            for k in data:
-                if k.startswith('db'):
-                    self.definitions['keys_redis']['lines'].append([k, None, 'absolute'])
-
         return True
 
     def _check_raw_data(self, data):
